@@ -1,10 +1,12 @@
 # repoman: Checks
-# Copyright 2007-2012 Gentoo Foundation
+# Copyright 2007-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 """This module contains functions used in Repoman to ascertain the quality
 and correctness of an ebuild."""
 
+from __future__ import unicode_literals
+
 import codecs
 from itertools import chain
 import re
@@ -480,6 +482,7 @@ class InheritEclass(LineCheck):
 			self._disabled = any(x in inherited for x in self._exempt_eclasses)
 		else:
 			self._disabled = False
+		self._eapi = pkg.eapi
 
 	def check(self, num, line):
 		if not self._inherit:
@@ -488,10 +491,14 @@ class InheritEclass(LineCheck):
 			if self._disabled or self._ignore_missing:
 				return
 			s = self._func_re.search(line)
-			if s:
-				self._func_call = True
-				return '%s.eclass is not inherited, but "%s" found at line: %s' % \
-					(self._eclass, s.group(3), '%d')
+			if s is not None:
+				func_name = s.group(3)
+				eapi_func = _eclass_eapi_functions.get(func_name)
+				if eapi_func is None or not eapi_func(self._eapi):
+					self._func_call = True
+					return ('%s.eclass is not inherited, '
+						'but "%s" found at line: %s') % \
+						(self._eclass, func_name, '%d')
 		elif not self._func_call:
 			self._func_call = self._func_re.search(line)
 
@@ -500,6 +507,10 @@ class InheritEclass(LineCheck):
 			self.repoman_check_name = 'inherit.unused'
 			yield 'no function called from %s.eclass; please drop' % self._eclass
 
+_eclass_eapi_functions = {
+	"usex" : lambda eapi: eapi not in ("0", "1", "2", "3", "4", "4-python", "4-slot-abi")
+}
+
 # eclasses that export ${ECLASS}_src_(compile|configure|install)
 _eclass_export_functions = (
 	'ant-tasks', 'apache-2', 'apache-module', 'aspell-dict',
@@ -808,7 +819,8 @@ class PortageInternalVariableAssignment(LineCheck):
 _base_check_classes = (InheritEclass, LineCheck, PhaseCheck)
 _constant_checks = tuple(chain((v() for k, v in globals().items()
 	if isinstance(v, type) and issubclass(v, LineCheck) and v not in _base_check_classes),
-	(InheritEclass(k, **kwargs) for k, kwargs in _eclass_info.items())))
+	(InheritEclass(k, **portage._native_kwargs(kwargs))
+	for k, kwargs in _eclass_info.items())))
 
 _here_doc_re = re.compile(r'.*\s<<[-]?(\w+)$')
 _ignore_comment_re = re.compile(r'^\s*#')
@@ -871,17 +883,18 @@ def run_checks(contents, pkg):
 				multiline = line
 				continue
 
-		# Finally we have a full line to parse.
-		is_comment = _ignore_comment_re.match(line) is not None
-		for lc in checks:
-			if is_comment and lc.ignore_comment:
-				continue
-			if lc.check_eapi(pkg.eapi):
-				ignore = lc.ignore_line
-				if not ignore or not ignore.match(line):
-					e = lc.check(num, line)
-					if e:
-						yield lc.repoman_check_name, e % (num + 1)
+		if not line.endswith("#nowarn\n"):
+			# Finally we have a full line to parse.
+			is_comment = _ignore_comment_re.match(line) is not None
+			for lc in checks:
+				if is_comment and lc.ignore_comment:
+					continue
+				if lc.check_eapi(pkg.eapi):
+					ignore = lc.ignore_line
+					if not ignore or not ignore.match(line):
+						e = lc.check(num, line)
+						if e:
+							yield lc.repoman_check_name, e % (num + 1)
 
 	for lc in checks:
 		i = lc.end()
